<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Relations\HasOne;
use Illuminate\Support\Collection;

# use Illuminate\Database\Eloquent\Model;

class Client extends Model
{
    protected $table = 'client';

    public function displayName()
    {
        return $this->name_last . ', ' . $this->name_first;
    }

    public function mcp()
    {
        return $this->hasOne(Pro::class, 'id', 'mcp_pro_id');
    }

    public function pcp()
    {
        return $this->hasOne(Pro::class, 'id', 'physician_pro_id');
    }

    public function cm()
    {
        return $this->hasOne(Pro::class, 'id', 'cm_pro_id');
    }

    public function rmm()
    {
        return $this->hasOne(Pro::class, 'id', 'rmm_pro_id');
    }

    public function rme()
    {
        return $this->hasOne(Pro::class, 'id', 'rme_pro_id');
    }

    public function rms()
    {
        return $this->hasOne(Pro::class, 'id', 'rms_pro_id');
    }

    public function rmg()
    {
        return $this->hasOne(Pro::class, 'id', 'rmg_pro_id');
    }

    public function defaultNaPro()
    {
        return $this->hasOne(Pro::class, 'id', 'default_na_pro_id');
    }

    public function prosInMeetingWith()
    {
        return Pro::where('in_meeting_with_client_id', $this->id)->get();
    }

    public function notes()
    {
        return $this->hasMany(Note::class, 'client_id', 'id')
            ->orderBy('effective_dateest', 'desc');
    }

    public function notesAscending()
    {
        return $this->hasMany(Note::class, 'client_id', 'id')
            ->orderBy('effective_dateest', 'asc');
    }

    public function activeNotes()
    {
        return $this->hasMany(Note::class, 'client_id', 'id')
            ->where('is_cancelled', false)
            ->orderBy('effective_dateest', 'desc');
    }

    public function cancelledNotes()
    {
        return $this->hasMany(Note::class, 'client_id', 'id')
            ->where('is_cancelled', true)
            ->orderBy('effective_dateest', 'desc');
    }

    public function sections()
    {
        return $this->hasMany(Section::class, 'client_id', 'id')
            ->where('is_active', true)
            ->orderBy('created_at', 'asc');
    }

    public function handouts()
    {
        $mappings = HandoutClient::where('client_id', $this->id)->get();
        $handouts = new Collection();
        foreach ($mappings as $mapping) {
            $handout = Handout::where('id', $mapping->handout_id)->first();
            $handout->handout_client_uid = $mapping->uid;
            $handouts->add($handout);
        }
        $handouts = $handouts->sortBy('created_at');
        return $handouts;
    }

    public function duplicateOf()
    {
        return $this->hasOne(Client::class, 'id', 'duplicate_of_client_id');
    }

    public function actionItems()
    {
        return $this->hasMany(ActionItem::class, 'client_id', 'id')
            ->orderBy('action_item_category', 'asc')
            ->orderBy('created_at', 'desc');
    }

    public function infoLines()
    {
        return $this->hasMany(ClientInfoLine::class, 'client_id', 'id')->orderBy('created_at', 'desc');
    }

    public function measurements()
    {
        return $this->hasMany(Measurement::class, 'client_id', 'id')
            /*->distinct('label')*/
            ->where('is_removed', false)
            ->orderByRaw('ts DESC NULLS LAST');
    }

    public function nonZeroMeasurements()
    {
        return $this->hasMany(Measurement::class, 'client_id', 'id')
            /*->distinct('label')*/
            ->where('is_removed', false)
            ->where('is_cellular_zero', false)
            ->orderBy('effective_date', 'desc');
    }

    public function getNonZeroBpMeasurements(){
        return $this->hasMany(Measurement::class, 'client_id', 'id')
            /*->distinct('label')*/
            ->where('is_removed', false)
            ->where('label', '=', 'BP')
            ->where('sbp_mm_hg', '>', 0)
            ->where('dbp_mm_hg', '>', 0)
            ->orderBy('ts', 'desc');
    }

    public function getNonZeroWeightMeasurements(){
        return $this->hasMany(Measurement::class, 'client_id', 'id')
            /*->distinct('label')*/
            ->where('is_removed', false)
            ->where('label', '=', 'Wt. (lbs.)')
            ->where('numeric_value', '>', 0)
            ->orderBy('ts', 'desc');
    }

    public function currentCareMonth()
    {
        $cmStartDate = strtotime(date('Y-m-d'));
        $month = date("n", $cmStartDate);
        $year = date("Y", $cmStartDate);
        return CareMonth
            ::where('client_id', $this->id)
            ->whereRaw('EXTRACT(MONTH FROM start_date) = ?', [$month])
            ->whereRaw('EXTRACT(YEAR FROM start_date) = ?', [$year])
            ->first();
    }

    public function measurementsInCareMonth(CareMonth $careMonth)
    {
        $cmStartDate = strtotime($careMonth->start_date);
        $month = date("n", $cmStartDate);
        $year = date("Y", $cmStartDate);
        $measurements = Measurement
            ::where('client_id', $this->id)
            ->whereRaw('EXTRACT(MONTH FROM effective_date) = ?', [$month])
            ->whereRaw('EXTRACT(YEAR FROM effective_date) = ?', [$year])
            ->where('is_removed', false)
            ->orderBy('effective_date', 'desc')
            ->get();
        return $measurements;
    }

    public function allMeasurements()
    {
        return $this->hasMany(Measurement::class, 'client_id', 'id')
            ->where('is_removed', false)
            ->whereNull('parent_measurement_id')
            ->orderBy('label', 'asc')
            ->orderBy('effective_date', 'desc');
    }

    public function smses()
    {
        return $this->hasMany(ClientSMS::class, 'client_id', 'id')
            ->orderBy('created_at', 'desc');
    }

    public function documents()
    {
        return $this->hasMany(ClientDocument::class, 'client_id', 'id')
            ->orderBy('created_at', 'desc');
    }

    public function incomingReports() {
        return $this->hasMany(IncomingReport::class, 'client_id', 'id')
            ->orderBy('created_at', 'desc');
    }

    public function smsNumbers() {
        return $this->hasMany(ClientSMSNumber::class, 'client_id', 'id')
            ->orderBy('created_at', 'desc');
    }

    public function nextMcpAppointment()
    {
        if ($this->mcp) {
            return Appointment::where('client_id', $this->id)
                ->where('pro_id', $this->mcp->id)
                ->where('start_time', '>=', date('Y-m-d'))
                ->orderBy('start_time', 'asc')
                ->first();
        }
        return false;
    }

    public function appointments()
    {
        return $this->hasMany(Appointment::class, 'client_id', 'id')
            ->orderBy('start_time', 'desc');
    }

    public function appointmentsForProByStatus($forPro = 'all', $status = 'all')
    {
        $appointments = Appointment::where('client_id', $this->id);
        if($forPro !== 'all') {
            $forPro = Pro::where('uid', $forPro)->first();
            $appointments = $appointments->where('pro_id', $forPro->id);
        }
        if($status !== 'ALL') {
            $appointments = $appointments->where('status', $status);
        }
        $appointments = $appointments->orderBy('start_time', 'desc');
        return $appointments->get();
    }

    public function upcomingAppointments()
    {
        return $this->hasMany(Appointment::class, 'client_id', 'id')
            // ->where('raw_start_time', '>', date('Y-m-d H:i:s'))
            ->whereIn('status', ['CREATED', 'CONFIRMED'])
            ->orderBy('start_time', 'desc');
    }

    public function appointmentsFromLastWeek()
    {

        $dateLastWeek = date_sub(date_create(), date_interval_create_from_date_string("7 days"));
        $dateLastWeek = date_format($dateLastWeek, "Y-m-d");
        return $this->hasMany(Appointment::class, 'client_id', 'id')
            ->where('raw_date', '>=', $dateLastWeek)
            ->whereIn('status', ['CREATED', 'CONFIRMED'])
            ->orderBy('start_time', 'desc');
    }

    public function memos()
    {
        return $this->hasMany(ClientMemo::class, 'client_id', 'id')
            ->where('is_cancelled', false)
            ->orderBy('created_at', 'desc');
    }

    public function devices()
    {
        return $this->hasMany(ClientBDTDevice::class, 'client_id', 'id')
            ->where('is_active', true)
            ->orderBy('created_at', 'desc');
    }

    public function deactivatedDevices()
    {
        return $this->hasMany(ClientBDTDevice::class, 'client_id', 'id')
            ->where('is_active', false)
            ->orderBy('created_at', 'desc');
    }

    public function hasDevice($_device)
    {
        $count = ClientBDTDevice::where('client_id', $this->id)
            ->where('device_id', $_device->id)
            ->where('is_active', true)
            ->count();
        return !!$count;
    }

    public function deviceMeasurements()
    {
        return $this->hasMany(ClientBDTMeasurement::class, 'client_id', 'id')
            ->orderBy('created_at', 'desc');
    }

    public function activeMcpRequest()
    {
        return $this->hasOne(McpRequest::class, 'id', 'active_mcp_request_id');
    }

    public function clientPrograms()
    {
        return $this->hasMany(ClientProgram::class, 'client_id', 'id')
            ->where('is_active', true)
            ->orderBy('title', 'desc');
    }

    public function tickets()
    {
        return $this->hasMany(Ticket::class, 'client_id', 'id')
            ->orderBy('is_open', 'desc')
            ->orderBy('created_at', 'desc');
    }

    public function mcpDisplayName()
    {
    }

    public function rmeDisplayName()
    {
    }

    public function firstCellularBPDevice()
    {
        $devices = $this->devices();
        $x = null;
        foreach($devices as $device){
            if($device->device->category == 'BP'){
                $x = $device;
                continue;
            }
        }
        return $x;
    }

    public function getFirstCellularBPMeasurementAt()
    {
    }

    public function getLatestCellularBPMeasurementAt()
    {
    }

    public function getTotalCellularBPMeasurements()
    {
    }

    public function firstCellularWeightDevice()
    {

    }

    public function getFirstCellularWeightMeasurementAt()
    {
    }

    public function getLatestCellularWeightMeasurementAt()
    {
    }

    public function getTotalCellularWeightMeasurements()
    {
    }

    public function prosWithAccess()
    {

        $pros = [];

        // directly associated pros
        $pro = $this->mcp;
        if ($pro && $pro->id) $pros[] = ["pro" => $pro->displayName(), "association" => 'MCP'];
        $pro = $this->pcp;
        if ($pro && $pro->id) $pros[] = ["pro" => $pro->displayName(), "association" => 'PCP (Physician)'];
        $pro = $this->cm;
        if ($pro && $pro->id) $pros[] = ["pro" => $pro->displayName(), "association" => 'CM'];
        $pro = $this->rmm;
        if ($pro && $pro->id) $pros[] = ["pro" => $pro->displayName(), "association" => 'RMM'];
        $pro = $this->rme;
        if ($pro && $pro->id) $pros[] = ["pro" => $pro->displayName(), "association" => 'RME'];

        // via client pro access
        $cpAccesses = ClientProAccess::where('client_id', $this->id)->where('is_active', true)->get();
        foreach ($cpAccesses as $cpAccess) {
            if (!$cpAccess->pro) continue;
            $pros[] = ["pro" => $cpAccess->pro->displayName(), "association" => 'Via Client-Pro Access'];
        }

        // via appointments
        $appointments = Appointment::where('client_id', $this->id)->get();
        foreach ($appointments as $appointment) {
            if (!$appointment->pro) continue;
            $pros[] = ["pro" => $appointment->pro->displayName(), "association" => 'Via Appointment: ' . $appointment->raw_date];
        }

        // via client program
        $clientPrograms = ClientProgram::where('client_id', $this->id)->where('is_active', true)->get();
        foreach ($clientPrograms as $clientProgram) {
            if ($clientProgram->mcp)
                $pros[] = ["pro" => $clientProgram->mcp->displayName(), "association" => 'Program MCP: ' . $clientProgram->title];
            if ($clientProgram->manager)
                $pros[] = ["pro" => $clientProgram->manager->displayName(), "association" => 'Program Manager: ' . $clientProgram->title];
        }

        // sort by pro name
        $name = array_column($pros, 'pro');
        array_multisort($name, SORT_ASC, $pros);

        return $pros;

    }

    public function mcpRequests()
    {
        return $this->hasMany(McpRequest::class, 'for_client_id', 'id')
            ->orderBy('created_at', 'desc');
    }

    public function eligibleRefreshes()
    {
        return $this->hasMany(ClientEligibleRefresh::class, 'client_id', 'id')
            ->orderBy('created_at', 'desc');
    }

    public function mbPayerValidationResults()
    {
        return $this->hasMany(ClientMBPayerValidationResult::class, 'client_id', 'id')
            ->orderBy('created_at', 'desc');
    }

    public function payer()
    {
        return $this->hasOne(MBPayer::class, 'id', 'mb_payer_id');
    }

    public function supplyOrders()
    {
        return $this->hasMany(SupplyOrder::class, 'client_id', 'id')
            ->where('is_cancelled', false)
            ->orderBy('created_at', 'desc');
    }

    public function readyToShipSupplyOrders()
    {
        return $this->hasMany(SupplyOrder::class, 'client_id', 'id')
            ->where('is_cancelled', false)
            ->where('is_cleared_for_shipment', true)
            ->whereNull('shipment_id')
            ->orderBy('created_at', 'desc');
    }

    public function shipments()
    {
        return $this->hasMany(Shipment::class, 'client_id', 'id')
            ->where('is_cancelled', false)
            ->orderBy('created_at', 'desc');
    }

    public function numSignedNotes() {
        return Note::where('client_id', $this->id)
            ->where('is_cancelled', false)
            ->where('is_signed_by_hcp', true)
            ->count();

    }

}
